package cz.fidentis.analyst.data.face;

import cz.fidentis.analyst.data.face.impl.HumanFaceImpl;
import cz.fidentis.analyst.data.mesh.MeshModel;

import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;

/**
 * This interface provides services that instantiates a human face.
 *
 * @author Radek Oslejsek
 */
public interface HumanFaceFactory {

    /**
     * Reads a 3D human face from the given OBJ file.
     *
     * @param file OBJ file
     * @param loadLandmarks If {@code true}, then the constructor aims to load landmarks as well
     * @throws IOException on I/O failure
     */
    static HumanFace create(File file, boolean loadLandmarks) throws IOException {
        return new HumanFaceImpl(file, loadLandmarks);
    }

    /**
     * Reads a 3D human face from the given OBJ file.
     *
     * @param file OBJ file
     * @throws IOException on I/O failure
     */
    static HumanFace create(File file) throws IOException {
        return create(file, true);
    }

    /**
     * Recover human face from serialized dump file. As the dump file can contain additional
     * objects, e.g., camera, it is supposed that the given input stream is set to a human face object.
     *
     * @param in A dump file as object input stream
     * @return The instance of the human face
     * @throws IOException If reading fails
     * @throws ClassNotFoundException If there is no human face in the stream
     */
    static HumanFace create(ObjectInputStream in) throws IOException, ClassNotFoundException {
        return (HumanFace) in.readObject();
    }

    /**
     * Creates a human face from existing mesh model. The mesh model is stored directly (not copied).
     *
     * @param model Mesh model
     * @param id Canonical path to the OBJ file
     * @throws IllegalArgumentException if the {@code model} is {@code null}
     */
    static HumanFace create(MeshModel model, String id) {
        return new HumanFaceImpl(model , id);
    }
}
