
__attribute__ ((reqd_work_group_size(MAX_GROUP_SIZE, 1, 1)))
kernel void copyAndMergeBoundingBoxes(const int size, global Point3D* vertices, global BoundingBox* bounds){
    int threadID = get_global_id(0);
    short localID = get_local_id(0);

    local BoundingBox boxes[MAX_GROUP_SIZE];
    boxes[localID] = threadID < size ? getVertexBBox(vertices[threadID]) : getMaxBoundingBox();
    barrier(CLK_LOCAL_MEM_FENCE);

    for(short groupShift = MAX_GROUP_SIZE>>1; groupShift>=1; groupShift>>=1){
        if(localID<groupShift){
            boxes[localID] = updateBBox(boxes[localID],boxes[localID+groupShift]);
        }
        barrier(CLK_LOCAL_MEM_FENCE);
    }

    if(localID == 0.0f){
        bounds[get_group_id(0)] = boxes[0];
    }
}

__attribute__ ((reqd_work_group_size(WARP_GROUP_SIZE, 1, 1)))
kernel void mergeBoundingBoxes(const int size, const int shift, global BoundingBox* bounds){
    int threadID = get_global_id(0) * shift;
    short localID = get_local_id(0);
    local BoundingBox boxes[WARP_GROUP_SIZE];
    boxes[localID] = threadID < size ? bounds[threadID] : getMaxBoundingBox();

    barrier(CLK_LOCAL_MEM_FENCE);

    for(short groupShift = WARP_GROUP_SIZE>>1; groupShift>=1; groupShift>>=1){
        if(localID<groupShift){
            boxes[localID] = updateBBox(boxes[localID],boxes[localID+groupShift]);
        }
        barrier(CLK_LOCAL_MEM_FENCE);
    }

    if(localID == 0.0f){
        bounds[threadID] = boxes[0];
    }
}

__attribute__ ((reqd_work_group_size(WARP_GROUP_SIZE, 1, 1)))
kernel void getLargestInteger(const int shift, const int leafCount, global int* leaves){
    int threadID = get_global_id(0) * shift;
    short localID = get_local_id(0);

    local int localLeaves[WARP_GROUP_SIZE];
    localLeaves[localID] = threadID < leafCount ? leaves[threadID] : 0;
    barrier(CLK_LOCAL_MEM_FENCE);

    if(localID<16.0f){
        localLeaves[localID] = max(localLeaves[localID], localLeaves[localID+16]);
        barrier(CLK_LOCAL_MEM_FENCE);
        localLeaves[localID] = max(localLeaves[localID], localLeaves[localID+8]);
        barrier(CLK_LOCAL_MEM_FENCE);
        localLeaves[localID] = max(localLeaves[localID], localLeaves[localID+4]);
        barrier(CLK_LOCAL_MEM_FENCE);
        localLeaves[localID] = max(localLeaves[localID], localLeaves[localID+2]);
        barrier(CLK_LOCAL_MEM_FENCE);
        localLeaves[localID] = max(localLeaves[localID], localLeaves[localID+1]);
        barrier(CLK_LOCAL_MEM_FENCE);
        if(localID == 0.0f){
            leaves[threadID] = localLeaves[0];
        }
    }
}

__attribute__ ((reqd_work_group_size(WARP_GROUP_SIZE, 1, 1)))
kernel void initializeMemory(const int leavesSize, const int value, global int* leaves){
    int threadID = get_global_id(0);
    if(threadID < leavesSize){
        leaves[threadID] = value;
    }
}