#define STACK_SIZE 64

// This file computes closest ray intersection with octree in 1 thread per ray manner.
// author Marek Horský

inline struct DirectionInfo getDirection(Point3D lowerBound, Point3D upperBound, Vector3D rd) {
    char subNodeMask = 0;
    Vector3D direction = rd;
    Vector3D scaledRD = direction * -10000.0f;
    Point3D originAdd = (Point3D) {0,0,0};

    if (rd.x < 0.0f) {
        originAdd = (Point3D) {lowerBound.x + upperBound.x,originAdd.y,originAdd.z};
        rd = (Vector3D) {rd.x*-1,rd.y,rd.z};
        subNodeMask |= 4;
    }
    if (rd.y < 0.0f) {
        originAdd = (Point3D) {originAdd.x,lowerBound.y + upperBound.y,originAdd.z};
        rd = (Vector3D) {rd.x,rd.y*-1,rd.z};
        subNodeMask |= 2;
    }
    if (rd.z < 0.0f) {
        originAdd = (Point3D) {originAdd.x,originAdd.y,lowerBound.z + upperBound.z};
        rd = (Vector3D) {rd.x,rd.y,rd.z*-1};
        subNodeMask |= 1;
    }

    return (struct DirectionInfo) {originAdd, direction, rd,scaledRD, subNodeMask,lowerBound,upperBound,
        1^subNodeMask,2^subNodeMask,3^subNodeMask,4^subNodeMask,5^subNodeMask,6^subNodeMask,7^subNodeMask};
}

// Method calculate closest intersections of rays with triangles loaded in octree

__attribute__ ((reqd_work_group_size(WARP_GROUP_SIZE, 1, 1)))
kernel void calculateClosestIntersections(const int rayCount, const int maxLeafSize,
      const int smoothing, const int filter,
      global Point3D* origins, global Vector3D* directions, // Ray direction
      global MeshTriangle* triangles, // Mesh Triangles - triangle == 6 float3 values stored continuously
      global Point3D* octreeBBox,
      global int* indices, global unsigned int* triangleIndices, // Index mappings - nodes to octants and leaves to triangles
      global float8* closestIntersections){ // Results --> Everything is casted to floats in float8* type to avoid waste of space during alignment. There is a room for better representation

      //Allocate Memory
      local struct DirectionInfo info[1];

      if(get_local_id(0)==0.0f){
          info[0] = getDirection(octreeBBox[0], octreeBBox[1], directions[0]);
      }

      OctNode stackMemory[STACK_SIZE];

      RayIntersection intersection = traverseOctree(rayCount, maxLeafSize,
                                    smoothing, //Smoothing enum
                                    filter, // Applied filter
                                    origins[get_global_id(0)], info, //Ray
                                    (OctNodeStack) {STACK_SIZE,-1, -1, stackMemory}, //Octree traversal stack
                                    indices, //Octree
                                    triangleIndices, triangles); //Mesh Triangles

      closestIntersections[get_global_id(0)] = (float8) {intersection.point.x,intersection.point.y,intersection.point.z,intersection.triangle,
                                          intersection.normal.x,intersection.normal.y,intersection.normal.z,intersection.distance};
}