package cz.fidentis.analyst.gui.project.faceinfo;

import cz.fidentis.analyst.gui.project.ProjectPanel;
import cz.fidentis.analyst.gui.project.ProjectPanelAction;
import cz.fidentis.analyst.gui.project.table.ProjectTable;
import java.awt.event.ActionEvent;
import javax.swing.JTabbedPane;

/**
 * Action listener for panel showing face details.
 *
 * @author Radek Oslejsek
 */
public class FacesInfoAction extends ProjectPanelAction<FaceInfoControlPanel> {
    
    /**
     * Constructor for project window when project panel is provided
     *
     * @param projectPanel Project panel
     * @param topControlPane Top component for placing control panels
     */
    public FacesInfoAction(ProjectPanel projectPanel, JTabbedPane topControlPane) {
        super(projectPanel, topControlPane);
        
        setControlPanel(new FaceInfoControlPanel(this));

        setShowHideCode(
                () -> { // SHOW
                },
                () -> { // HIDE
                }
        );        
    }
    
    @Override
    public void actionPerformed(ActionEvent ae) {
        String action = ae.getActionCommand();
        
        switch (action) {
            case FaceInfoControlPanel.ACTION_LOAD_GEOMETRY_INFO:
                int faceIndex = getProjectPanel().getProject().getFacesProxy().getFaceIndex(getSelectedFaceName());
                if (faceIndex >= 0 && !getProjectPanel().getProject().getFacesProxy().isInMemory(faceIndex)) {
                    getProjectPanel().getProject().getFacesProxy().getFace(faceIndex); // load
                    getControlPanel().getInfoPanel().showFaceState( // show info
                            getProjectPanel().getProject().getFacesProxy(),
                            getSelectedFaceName()
                    );
                }
                break;
            case ProjectPanel.ACTION_FACES_SELECTED:
                getControlPanel().getInfoPanel().showFaceState(
                        getProjectPanel().getProject().getFacesProxy(),
                        getSelectedFaceName()
                );
                break;
            default:
        }
    }
    
    private String getSelectedFaceName() {
        ProjectTable table =  getProjectPanel().getProjectTable();
        if (table.getSelectedRowCount() == 1) {
            int row = table.getSelectedRow();
            return (String) table.getValueAt(row, 1);
        } else {
            return null;
        }
    }
}
