package cz.fidentis.analyst.gui.task.curvature;

import cz.fidentis.analyst.gui.task.ControlPanel;

import java.awt.*;
import java.awt.event.ActionListener;
import java.util.List;
import javax.swing.*;

/**
 * Control panel for curvature analysis.
 * 
 * @author Radek Oslejsek
 */
public class CurvaturePanel extends ControlPanel {
    
    /*
     * Mandatory design elements
     */
    public static final String ICON = "curvature28x28.png";
    public static final String NAME = "Curvature";
    
    /*
     * Actions triggered by this panel and served by the associated action listener 
     */
    //public static final String ACTION_COMMAND_SHOW_HIDE_HEATMAP = "show-hide heatmap";
    public static final String ACTION_COMMAND_SET_CURVATURE_TYPE = "set curvature type";
    
    /*
     * Configuration of panel-specific GUI elements
     */
    public static final String GAUSSIAN_CURVATURE = "Gaussian";
    public static final String MEAN_CURVATURE = "Mean";
    public static final String MIN_CURVATURE = "Min";
    public static final String MAX_CURVATURE = "Max";
    
    /**
     * Constructor.
     * @param action Action listener
     */
    public CurvaturePanel(ActionListener action) {
        super(action);

        setLayout(new GridBagLayout());
        setName(NAME);

        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(20, 10, 20, 0);
        c.gridx = 0;
        c.gridy = 0;
        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.NONE;

        JLabel label = new JLabel("Curvature:");
        label.setFont(new Font("Arial", 1, 15));
        add(label, c);
        c.gridx++;

        JComboBox cbox = new JComboBox(List.of(GAUSSIAN_CURVATURE, MEAN_CURVATURE, MIN_CURVATURE, MAX_CURVATURE).toArray(String[]::new));
        cbox.addActionListener(createListener(action, ACTION_COMMAND_SET_CURVATURE_TYPE));
        add(cbox, c);

        c.gridy++;
        c.gridx = 0;
        c.gridwidth = 2;
        var todoLabel = new JLabel("Here will be an interactive histogram (under construction)");
        todoLabel.setForeground(new java.awt.Color(255, 153, 0));
        add(todoLabel, c);
        

        // horizontal strut
        c.weightx = 1.0;
        c.gridheight = 1;
        c.gridx += 2;
        add(Box.createVerticalStrut(5), c);

        // vertical strut
        c.weighty = 1.0;
        c.gridy++;
        //c.gridx++;
        c.anchor = GridBagConstraints.CENTER;
        c.fill = GridBagConstraints.BOTH;
        add(Box.createHorizontalStrut(5), c);

        //builder.addVerticalStrut();
    }

    @Override
    public ImageIcon getIcon() {
        return getStaticIcon();
    }
    
    /**
     * Static implementation of the {@link #getIcon()} method.
     * 
     * @return Control panel icon
     */
    public static ImageIcon getStaticIcon() {
        return new ImageIcon(CurvaturePanel.class.getClassLoader().getResource("/" + ICON));
    }
}
