package cz.fidentis.analyst.gui.task.faceinfo;

import cz.fidentis.analyst.gui.task.ControlPanel;
import cz.fidentis.analyst.project.FacesProxy;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;

/**
 * A control panel wrapper for the {@code FaceInfoPanel}
 * @author Matej Kovar
 * @author Radek Oslejsek
 */
public class FaceOverviewPanel extends ControlPanel {
    
    /*
     * Mandatory design elements
     */
    public static final String NAME = "Face Info";
    public static final String DEFAULT_ICON = "head28x28.png";
    public static final String PRIMARY_FACE_ICON = "primaryFace28x28.png";
    public static final String SECONDARY_FACE_ICON = "secondaryFace28x28.png";
    
    /*
     * Actions triggered by this panel and served by the associated action listener 
     */
    // NONE
    
    /**
     * Faces processed by current analytical task
     */
    private FacesProxy faces;
    
    /**
     * Face to be shown
     */
    private int faceIndex = -1;
    
    /**
     * Constructor.
     * @param action Action listener
     * @param faces Faces processed by current analytical task
     * @param faceIndex Face to be shown, -1 = none 
     */
    public FaceOverviewPanel(ActionListener action, FacesProxy faces, int faceIndex) {
        super(action);
        this.setName(NAME);
        this.faces = faces;
        this.faceIndex = faceIndex;
        
        initComponents();
        
        facePanel1.showFaceState(faces, faceIndex);
    }
    
    @Override
    public ImageIcon getIcon() {
        switch (this.faceIndex) {
            case 0: // primary face
                return new ImageIcon(FaceOverviewPanel.class.getClassLoader().getResource("/" + PRIMARY_FACE_ICON));
            case 1: // decondary face
                return new ImageIcon(FaceOverviewPanel.class.getClassLoader().getResource("/" + SECONDARY_FACE_ICON));
            default: // default (no face selected)
                return new ImageIcon(FaceOverviewPanel.class.getClassLoader().getResource("/" + DEFAULT_ICON));
        }        
    }    

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        facePanel1 = new cz.fidentis.analyst.gui.elements.FaceInfoPanel();

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(facePanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(facePanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private cz.fidentis.analyst.gui.elements.FaceInfoPanel facePanel1;
    // End of variables declaration//GEN-END:variables

}
