package cz.fidentis.analyst.gui.task.interactivemask;

import cz.fidentis.analyst.canvas.Canvas;
import cz.fidentis.analyst.engines.face.events.HumanFaceSelectedEvent;
import cz.fidentis.analyst.data.face.HumanFaceEvent;
import cz.fidentis.analyst.data.face.HumanFaceListener;
import cz.fidentis.analyst.gui.task.ControlPanelAction;
import cz.fidentis.analyst.project.FacesProxy;
import cz.fidentis.analyst.data.ray.RayIntersection;

import javax.swing.*;
import java.awt.event.ActionEvent;

/**
 *
 * @author Mario Chromik
 */
public class InteractiveMaskAction extends ControlPanelAction<InteractiveMaskPanel> implements HumanFaceListener {
    
    private String mode = InteractiveMaskPanel.ACTION_VIEW;
    
    /**
     * Constructor 
     * A new {@code InteractiveMaskPanel} is instantiated and added to the {@code topControlPane}
     * 
     * @param canvas OpenGLCanvas
     * @param faces Faces processed by current analytical task
     * @param topControlPane A top component when a new control panel is placed
     */
    public InteractiveMaskAction(Canvas canvas, FacesProxy faces, JTabbedPane topControlPane) {
        super(canvas, faces, topControlPane);
        setControlPanel(new InteractiveMaskPanel(this));
        setShowHideCode(
                () -> { // SHOW
                    getCanvas().getScene().showInteractiveMask(getCanvas().getScene().getPrimaryFaceSlot(), true);
                    getCanvas().getScene().showInteractiveMask(getCanvas().getScene().getSecondaryFaceSlot(), true);
                },
                () -> { // HIDE
                    getCanvas().getScene().showInteractiveMask(getCanvas().getScene().getPrimaryFaceSlot(), false);
                    getCanvas().getScene().showInteractiveMask(getCanvas().getScene().getSecondaryFaceSlot(), false);
                }
        );
        getPrimaryDrawableFace().getHumanFace().registerListener(this);
        if (getSecondaryDrawableFace() != null) {
            getSecondaryDrawableFace().getHumanFace().registerListener(this);
        }
    }

    @Override
    public void actionPerformed(ActionEvent ae) {
         String action = ae.getActionCommand();
        
        switch (action) {
            case InteractiveMaskPanel.ACTION_DRAW:
                mode = InteractiveMaskPanel.ACTION_DRAW;
                break;
            case InteractiveMaskPanel.ACTION_EDIT:
                mode = InteractiveMaskPanel.ACTION_EDIT;
                break;
            default:
                mode = InteractiveMaskPanel.ACTION_VIEW;
        }
    }

    @Override
    public void acceptEvent(HumanFaceEvent event) {
        if (event instanceof HumanFaceSelectedEvent huEvent) {
            if (huEvent.getFace().equals(getPrimaryFace())
                    && getPrimaryInteractiveMask() == null) {
                getScene().setDrawableInteractiveMask(getScene().getPrimaryFaceSlot(), 
                        huEvent.getFace());
            } 
            if(huEvent.getFace().equals(getSecondaryFace()) && 
                    getSecondaryInteractiveMask() == null) {
                getScene().setDrawableInteractiveMask(getScene().getSecondaryFaceSlot(), 
                        huEvent.getFace());
            }
            
            RayIntersection closestIntersection = huEvent.getIntersection();
            if (mode.equals(InteractiveMaskPanel.ACTION_DRAW) && closestIntersection != null && huEvent.getFace().getSurfaceMask() != null) {
                huEvent.getFace().getSurfaceMask().addPoint(
                        huEvent.getIntersection().getPosition(),
                        huEvent.getIntersection().getHitTriangle().getFacet()
                );
            }
            renderScene();
       }
    }
    
}
