package cz.fidentis.analyst.engines.avgmesh;

import cz.fidentis.analyst.data.mesh.MeshFacet;
import cz.fidentis.analyst.data.mesh.MeshModel;
import cz.fidentis.analyst.engines.avgmesh.impl.AvgMeshVisitorNN;
import cz.fidentis.analyst.engines.avgmesh.impl.AvgMeshVisitorRC;

import java.util.Collection;
import java.util.Collections;

/**
 * Strategies of the average mesh computation.
 *
 * @author Radek Oslejsek
 *
 * @param templateMesh Template mesh which is transformed to the averaged mesh.
 *                     The original mesh remains unchanged. New mesh is allocated instead.
 */
public record AvgMeshConfig(Collection<MeshFacet> templateMesh) {

    /**
     * Constructor.
     * @param facet Template mesh facet which is transformed to the averaged mesh.
     *              The original mesh remains unchanged. New mesh is allocated instead.
     */
    public AvgMeshConfig(MeshFacet facet) {
        this(Collections.singleton(facet));
    }

    /**
     * Constructor.
     * @param model Template mesh model which is transformed to the averaged mesh.
     *              The original mesh remains unchanged. New mesh is allocated instead.
     */
    public AvgMeshConfig(MeshModel model) {
        this(model.getFacets());
    }

    /**
     * Instantiates and returns a k-d tree visitor.
     * @return a visitor
     */
    public AvgMeshVisitor getNearestNeighborsVisitor() {
        return new AvgMeshVisitorNN(templateMesh);
    }

    /**
     * Instantiates and returns an octree visitor.
     * @return a visitor
     */
    public AvgMeshVisitor getRayCastingVisitor() {
        return new AvgMeshVisitorRC(templateMesh);
    }

}
