package cz.fidentis.analyst.engines.avgmesh;

import cz.fidentis.analyst.data.kdtree.KdTree;
import cz.fidentis.analyst.data.kdtree.KdTreeVisitor;
import cz.fidentis.analyst.data.mesh.MeshModel;
import cz.fidentis.analyst.data.octree.Octree;
import cz.fidentis.analyst.data.octree.OctreeVisitor;

/**
 * A functional interface declaring the ability to deform a template mesh into an average mesh based on visiting other meshes.
 *
 * @author Radek Oslejsek
 */
public interface AvgMeshVisitor extends KdTreeVisitor, OctreeVisitor {

    /**
     * Returns the averaged mesh model
     *
     * @return averaged mesh model
     */
    MeshModel getAveragedMeshModel();

    @Override
    default boolean isThreadSafe() {
        return false;
    }

    @Override
    default void visitKdTree(KdTree kdTree) {
        throw new UnsupportedOperationException();
    }

    @Override
    default void visitOctree(Octree octree) {
        throw new UnsupportedOperationException();
    }
}
