package cz.fidentis.analyst.engines.distance;

import cz.fidentis.analyst.data.mesh.MeshFacet;
import cz.fidentis.analyst.data.mesh.MeshModel;
import cz.fidentis.analyst.engines.distance.measurement.MeshDistances;

import java.util.Collection;
import java.util.Collections;

/**
 * Stateless services for mesh-to-mesh distance measurement.
 * It is a wrapper to the stateful implementation of {@link cz.fidentis.analyst.engines.distance.MeshDistanceVisitor}.
 *
 * @author Radek Oslejsek
 */
public interface MeshDistanceServices {

    /**
     * Returns distances between the source mesh stored in the strategy object and meshes provided as input parameter.
     *
     * @param facets Meshes to be checked
     * @param config Configuration of the service
     * @return Distance measurement
     */
    static MeshDistances measure(Collection<MeshFacet> facets, MeshDistanceConfig config) {
        var visitor = config.getVisitor();
        facets.forEach(facet -> facet.accept(visitor));
        visitor.dispose();
        return visitor.getDistancesOfVisitedFacets();
    }

    /**
     * Returns distances between the source mesh stored in the strategy object and the mesh provided as input parameter.
     *
     * @param facet Mesh to be checked
     * @param config Configuration of the service
     * @return Distance measurement
     */
    static MeshDistances measure(MeshFacet facet, MeshDistanceConfig config) {
        return measure(Collections.singleton(facet), config);
    }

    /**
     * Returns distances between the source mesh stored in the strategy object and the mesh provided as input parameter.
     *
     * @param meshModel Mesh to be checked
     * @param config Configuration of the service
     * @return Distance measurement
     */
    static MeshDistances measure(MeshModel meshModel, MeshDistanceConfig config) {
        return measure(meshModel.getFacets(), config);
    }
}
