package cz.fidentis.analyst.engines.symmetry;

import cz.fidentis.analyst.engines.sampling.PointSamplingConfig;
import cz.fidentis.analyst.engines.symmetry.impl.SymmetryVisitorMesh;
import cz.fidentis.analyst.engines.symmetry.impl.SymmetryVisitorRobust;
import cz.fidentis.analyst.engines.symmetry.impl.SymmetryVisitorRobustMesh;

/**
 * Symmetry estimation strategies sand their configuration.
 *
 * @author Radek Oslejsek
 *
 * @param method Symmetry estimation method
 * @param priSampling Primary sampling strategy used for finding candidate planes
 * @param secSampling Secondary sampling strategy used for finding the best candidate.
 */
public record SymmetryConfig(Method method, PointSamplingConfig priSampling, PointSamplingConfig secSampling) {

    /**
     * Simplified constructor for the same sampling strategies or for symmetry methods with only one sampling strategy..
     *
     * @param method Symmetry estimation method
     * @param strategy Sub-sampling strategy
     */
    public SymmetryConfig(Method method, PointSamplingConfig strategy) {
        this(method, strategy, strategy);
    }

    /**
     * Instantiates and returns a symmetry visitor.
     *
     * @return a symmetry visitor
     */
    public SymmetryVisitor getVisitor() {
        return switch (method) {
            case ROBUST_POINT_CLOUD -> new SymmetryVisitorRobust(priSampling, secSampling);
            case ROBUST_MESH -> new SymmetryVisitorRobustMesh(priSampling, secSampling);
            case OLD_MESH -> new SymmetryVisitorMesh(priSampling);
        };
    }

    /**
     * Symmetry method.
     *
     * @author Radek Oslejsek
     */
    public enum Method {
        ROBUST_POINT_CLOUD,
        ROBUST_MESH,
        OLD_MESH
    }
}
