package cz.fidentis.analyst.project;

import java.io.File;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * This class determine the structure of the JSON project configuration file.
 * It is used by Jackson serialization and deserialization.
 * All attributes have to have public getters and setters to be (de)serialized.
 * 
 * @author Radek Oslejsek
 */
public class ProjectJsonStructure {
    
    private List<Path> paths = new ArrayList<>();
    
    /**
     * Constructor from faces proxy used for serialization.
     * @param faces Faces of the project
     */
    public ProjectJsonStructure(FacesProxy faces) {
        this.paths = faces.getFaceFiles().stream()
                    .map(File::toPath)
                    .collect(Collectors.toList());
    }
    
    /**
     * An empty constructor required by Jackson deserialization.
     */
    public ProjectJsonStructure() {
    }
    
    /**
     * Returns a list of paths to project's faces
     * @return a list of paths to project's faces
     */
    public List<Path> getPaths() {
        return Collections.unmodifiableList(paths);
    }
    
    /**
     * Setter used by Jackson deserialization.
     * @param paths Faces of the project
     */
    public void setPaths(List<Path> paths) {
        this.paths = new ArrayList<>(paths);
    }
}
