# FIDENTIS Analyst 2

Software for processing and analysis of human faces. Re-implementation of [FIDENTIS Analyst](https://github.com/Fidentis/Analyst).

This project is being developed at Masaryk University, Brno, as collaborative project between the Human Computer Interaction Laboratory at Faculty of Informatics and the Department of Anthropology at Faculty of Science.

* **[User guide](https://gitlab.fi.muni.cz/grp-fidentis/analyst2/-/wikis/home)**

## Requirements (for development)

*  [Oracle JDK 17](https://www.oracle.com/java/technologies/downloads/#java17) or [OpenJDK 17](https://www.openlogic.com/openjdk-downloads)
*  [Maven](https://www.mkyong.com/maven/how-to-install-maven-in-windows/) for compilation

## Compilation
* Run `mvn clean install` or `mvn clean install -DskipTests` to build the Linux/Win application. The result is stored in 
  the `application/target/fidentisanalyst` folder and also in the distributable 
  `application/target/fidentisanalyst/FIDENTIS-Analyst-app-VERSION.zip` file.   
* Add the following Maven profile parameters to generate application bundles with JRE included in the application.   
  - `-Pcreate-linux-package`: Creates the `application/target/fidentisanalyst-linux` packed into the `application/target/linux-FIDENTIS-Analyst-app-VERSION.tar.gz` file. 
  - `-Pcreate-win32-package`: Creates the `application/target/fidentisanalyst-win` packed into the `application/target/win-FIDENTIS-Analyst-app-VERSION.zip` file.
  - `-Pcreate-win64-package`: Creates the `application/target/fidentisanalyst-win64` packed into the `application/target/win64-FIDENTIS-Analyst-app-VERSION.zip` file.
  - `-Pcreate-macosx-package`: Creates the `application/target/ios-FIDENTIS-Analyst-app-VERSION.dmg` file for Mac OS X.
  
## Running
* Either run a binary file located in the `application/target/fidentisanalyst/bin/` (JRE/JDK has to be installed in the system), or
* go to the `application` directory and run `mvn nbm:run-platform` (JRE/JDK has to be installed in the system), or
* run the `application/target/fidentisanalyst-[linux|win|win32]/bin/` application (bundled JRE is used).

## Developer's guide

* The source code has to be maintainable via Maven. If you use IDEA for development, be careful that this IDE transforms the original Maven project into its own obscure format. You can use it, but at the end, your code has to be installable and runnable via maven (see Installation and running). When pushing your code into gitlab, the maven installation is tested.
* Document public interfaces rigorously.
* Commit and push regularly. Keep branches (iterations) as short as possible.
* Write junit test whenever possible. For the `MeshModel` module, unit tests are mandatory for every new functionality! 
* The client part (GUI) of the project is developed in [NetBeans Platform](https://netbeans.apache.org/kb/docs/platform/index.html) framework.
  Some Swing components are generated by the NetBeans IDE GUI Designer. In this case, use NetBeans instead of IDEA to modify them.
* For further info, see the [project web page](https://www.radek-oslejsek.cz/it/fidentis-analyst-2/).

## Structure (modules):

*  **application:** From [NetBeans tutorial](https://platform.netbeans.org/tutorials/71/nbm-maven-quickstart.html): Enumerates included modules; permits interactive runs; produces various kinds of packaging, such as ZIP by default, but optionally JNLP, NBMs, and in the future perhaps OSGi; holds functional tests. Each module project specifies its own compile dependencies, while the appplication project's dependencies are anything additional that should be present at runtime. For example, that includes, by default, the whole platform cluster, that is, the set of modules constituting the "platform" cluster. The appplication project could include other clusters, or subsets of clusters, or whatever plugins you want included in your app which are not used as compilation dependencies. At a minimum, core.startup and its transitive dependencies are included.
*  **branding:** From [NetBeans tutorial](https://platform.netbeans.org/tutorials/71/nbm-maven-quickstart.html): This module project contains the global resources used for branding the application, such as the splash screen.
*  **Application modules (MeshModel, etc.):**  See the [project web page](https://www.radek-oslejsek.cz/it/fidentis-analyst-2/) for the architecture preview.
